@@ -1,15 +1,25 @@
 <?php
 /**
- * The Horde_Form_Renderer class provides HTML and other renderings of
- * forms for the Horde_Form:: package.
- *
  * Copyright 2001-2007 Robert E. Coyle <robertecoyle@hotmail.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
- * @author  Robert E. Coyle <robertecoyle@hotmail.com>
- * @package Form
+ * @author   Robert E. Coyle <robertecoyle@hotmail.com>
+ * @category Horde
+ * @license  http://www.horde.org/licenses/lgpl21 LGPL
+ * @package  Form
+ */
+
+/**
+ * The Horde_Form_Renderer class provides HTML and other renderings of
+ * forms for the Horde_Form:: package.
+ *
+ * @author    Robert E. Coyle <robertecoyle@hotmail.com>
+ * @category  Horde
+ * @copyright 2001-2007 Robert E. Coyle
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL
+ * @package   Form
  */
 class Horde_Form_Renderer {
 
@@ -22,6 +32,7 @@ class Horde_Form_Renderer {
     var $_varRenderer = null;
     var $_firstField = null;
     var $_stripedRows = false;
+    var $_forms = array();
 
     /**
      * Does the title of the form contain HTML? If so, you are responsible for
@@ -136,7 +147,7 @@ class Horde_Form_Renderer {
         $page->addInlineScript(
             sprintf('var sections_%1$s = new Horde_Form_Sections(\'%1$s\', \'%2$s\');',
                     $form->getName(),
-                    addslashes($open_section)));
+                    rawurlencode($open_section)));
 
         /* Loop through the sections and print out a tab for each. */
         echo "<div class=\"tabset\"><ul>\n";
@@ -185,6 +196,7 @@ class Horde_Form_Renderer {
 
     function close($focus = true)
     {
+        $this->_listAllFormVars();
         echo "</form>\n";
         if ($focus) {
             $GLOBALS['injector']
@@ -196,21 +208,51 @@ class Horde_Form_Renderer {
         }
     }
 
-    function listFormVars(&$form)
+    function getFormVars($form)
     {
         $variables = $form->getVariables(true, true);
         $vars = array();
         if ($variables) {
             foreach ($variables as $var) {
                 if (is_object($var)) {
-                    if (!$var->isReadonly()) {
-                        $vars[$var->getVarName()] = 1;
+                    // @todo H6 isHidden() is just a BC duct-tape because
+                    // hidden read-only fields had not been detected in the
+                    // past. The actual forms need to be fixed instead.
+                    if ($var->isHidden() || !$var->isReadonly()) {
+                        $varname = $var->getVarName();
+                        if (Horde_Array::getArrayParts($varname, $base, $keys)) {
+                            array_unshift($keys, $base);
+                            $place = &$vars;
+                            while ($key = array_shift($keys)) {
+                                if (!isset($place[$key])) {
+                                    $place[$key] = array();
+                                }
+                                $place = &$place[$key];
+                            }
+                            $place = 1;
+                        } else {
+                            $vars[$varname] = 1;
+                        }
                     }
                 } else {
                     $vars[$var] = 1;
                 }
             }
         }
+        return $vars;
+    }
+
+    function listFormVars(&$form)
+    {
+        echo '<input type="hidden" name="_formvars" value="' . htmlspecialchars(json_encode($this->getFormVars($form))) . '" />';
+    }
+
+    function _listAllFormVars()
+    {
+        $vars = array();
+        foreach ($this->_forms as $form) {
+            $vars = array_merge($vars, $this->getFormVars($form));
+        }
         echo '<input type="hidden" name="_formvars" value="' . htmlspecialchars(json_encode($vars)) . '" />';
     }
 
@@ -226,6 +268,8 @@ class Horde_Form_Renderer {
 
     function _renderForm(&$form, &$vars, $active)
     {
+        $this->_forms[] = $form;
+
         /* If help is present 3 columns are needed. */
         $this->_cols = $form->hasHelp() ? 3 : 2;
 
